#!/usr/bin/env python3

import sys
import os
import couchdb ## run in venv
import subprocess
def main():
    usr = os.environ.get('CAL_USR')
    pwd = os.environ.get('CAL_PWD')

    args = sys.argv

    if '--help' in args or "-h" in args:
        print("\n")
        print("Activate venv ($> source bin/activate) run:")
        print("$>  python auto-gen-ks.py --yn 2019-75036 [--show] [--srv localhost] [--compiler xelatex] [--db vl_db]")
        print("\n")
        sys.exit()

    if '--yn' in args:
        idx_yn = args.index('--yn') + 1
        yn = args[idx_yn]
    else:
        sys.exit("which ks? --yn (year-number) ")

    if '--srv' in args:
        idx_srv = args.index('--srv') + 1
        srv = args[idx_srv]
    else:
        srv = "localhost"

    if '--dbsrv' in args:
        idx_dbsrv = args.index('--dbsrv') + 1
        dbsrv = args[idx_dbsrv]
    else:
        dbsrv = "localhost"

    if '--compiler' in args:
        idx_comp = args.index('--comp') + 1
        comp = args[idx_comp]
    else:
        comp = "lualatex"

    if '--db' in args:
        idx_db = args.index('--db') + 1
        db = args[idx_db]
    else:
        db = "vl_db"

    if '--show' in args:
        show = args.index('--show')
    else:
        show = False

    if '--nocert' in args:
        nocert = True
    else:
        nocert = False

    if '--dcc' in args:
        dcc = True
    else:
        dcc = False

    ## tmp folder
    tmp_folder = "./tmp"

    ## files
    latex_file = "ks-{yn}.tex".format(yn=yn)
    pdf_file = "ks-{yn}.pdf".format(yn=yn)
    dcc_file = "dcc-{yn}.xml".format(yn=yn)

    ## doc
    cer_doc = "cer-{yn}".format(yn=yn)
    inter_yn = yn.replace("-", "_")

    ## data
    json_header = "Content-Type: application/json"
    post_data = "{\"save\":true}"
    xml_header = "Content-Type: text/xml"

    ## server
    cal_server = "http://{srv}:5001".format(srv=srv)
    xml_gen_server = "http://a73434:5006"
    xml_valid_server = "http://a73434:5005"
    ## url
    inter_url = "{server}/inter/{yn}".format(yn=inter_yn, server=cal_server)
    json_dcc_url  = "{server}/dcc/{yn}".format(yn=inter_yn, server=cal_server)
    latex_url = "{server}/cer/latex/{cer_doc}".format(server=cal_server, cer_doc=cer_doc)
    if usr:
        db_url = 'http://{usr}:{pwd}@{srv}:5984/'.format(srv=dbsrv, usr=usr, pwd=pwd)
    else:
        db_url = 'http://{srv}:5984/'.format(srv=dbsrv)
    dcc_url = "{server}/vl-dcc/{yn}".format(server=xml_gen_server, yn=yn)
    valid_url = "{server}/validate".format(server=xml_valid_server)

    ## commands
    cmd_cd = "cd {tmp}".format(tmp=tmp_folder)
    cmd_cert_doc = "curl --noproxy '*' --header '{header}'  -X POST  -d '{post_data}' {url}".format(header=json_header, post_data=post_data, url=inter_url)
    cmd_json_dcc_doc = "curl --noproxy '*'  --header '{header}'  -X POST  {url}".format(header=json_header, url=json_dcc_url)
    cmd_tex_file = "curl --noproxy '*' {url} > {tmp}/{file}".format(tmp=tmp_folder, url=latex_url, yn=yn, file=latex_file)
    cmd_compile_latex = "cd {tmp} && {comp} {file}".format(tmp=tmp_folder, comp=comp, file=latex_file)
    cmd_dcc_file = "curl --noproxy '*' {url} > {tmp}/{file}".format(tmp=tmp_folder, url=dcc_url, file=dcc_file)
    cmd_dcc_valid = "curl --noproxy '*' {url} --header '{header}' -X POST --data '@{tmp}/{file}'".format(tmp=tmp_folder, url=valid_url,header=xml_header, file=dcc_file)

    ## cdb inst.
    cdb = couchdb.Server(db_url)[db]

    if show:
        print("\n")
        print("\n")
        print("The files are dumped in {tmp}!".format(tmp=tmp_folder))
        print("\n")
        print("\n")
        print(" ⇨ DB document:\n{}\n".format(cer_doc))
        print(" ⇨ DB doc url:\n{}\n".format(cmd_cert_doc))
        print(" ⇨ DCC doc command:\n{}\n".format(cmd_json_dcc_doc))
        print(" ⇨ LaTeX file:\n{}\n".format(latex_file))
        print(" ⇨ LaTeX command:\n{}\n".format(cmd_tex_file))
        print(" ⇨ pdf file:\n{}\n".format(pdf_file))
        print(" ⇨ pdf command:\n{}\n".format(cmd_compile_latex))
        print(" ⇨ XML file:\n{}\n".format(dcc_file))
        print(" ⇨ XML generate file:\n{}\n".format(cmd_dcc_file))
        print(" ⇨ xml valid command:\n{}\n".format(cmd_dcc_valid))
        print(" ⇨ DCC url:\n{}\n".format(dcc_url))


    else:
        print("\n")
        print("\n")
        print("The files are dumped in {tmp}!".format(tmp=tmp_folder))
        print("\n")
        print("\n")
        if not nocert:
            ret = subprocess.getoutput(cmd_cert_doc)
            print("✓ cert document generated")

        ret = subprocess.getoutput(cmd_tex_file)
        print("✓ LaTeX file generated")
        ret = subprocess.getoutput(cmd_compile_latex)
        print("✓ compile LaTeX file 1st run")
        ret = subprocess.getoutput(cmd_compile_latex)
        print("✓ compile LaTeX file 2nd run")
        ret = subprocess.getoutput(cmd_compile_latex)
        print("✓ compile LaTeX file 3rd run")
        cdb.put_attachment(cdb[cer_doc], open("{}/{}".format(tmp_folder, latex_file), 'r', encoding="utf8"), filename=latex_file)
        print("✓ upload LaTeX to database ({})".format(latex_file))
        cdb.put_attachment(cdb[cer_doc], open("{}/{}".format(tmp_folder, pdf_file), 'rb'), filename=pdf_file)
        print("✓ upload pdf to database ({})".format(pdf_file))

        if dcc:
            ret = subprocess.getoutput(cmd_json_dcc_doc)
            print("✓ JSON DCC document generated")
            ret = subprocess.getoutput(cmd_dcc_file)
            print("✓ XML DCC document generated")
            print("↯ XML DCC document valid?")
            ret = subprocess.getoutput(cmd_dcc_valid)
            print(ret)
            cdb.put_attachment(cdb[cer_doc], open("{}/{}".format(tmp_folder, dcc_file), 'rb'), filename=dcc_file)
            print("✓ upload xml to database ({})".format(dcc_file))


if __name__ == "__main__":
    main()
