import sys
import os
import json
import couchdb

class DB:
    def __init__(self, config, app):

        usr = os.environ.get('CAL_USR')
        pwd = os.environ.get('CAL_PWD')
        host = config['db']['host']
        port = config['db']['port']
        db_name = config['db']['database']
        if usr and pwd:

            url = 'http://{usr}:{pwd}@{host}:{port}/'.format(usr=usr, pwd=pwd, host=host, port=port)

        else:
            url = 'http://{host}:{port}/'.format(host=host, port=port)
        srv = couchdb.Server(url)
        self.db = srv[db_name]
        self.config = config

    def get_doc(self, id):
        try:
            return self.db[id]
        except:
            return {'error':'Document not found'}

    def get_cal_docs(self, year_cert):
        """
            ..note::

                The descending=True option affects the order of the result tables
        """
        view_con = self.config.get('views').get('cal_docs')
        try:
            res = []
            # Ex.: http://a73434.berlin.ptb.de:5984/vl_db/_design/share/_view/Cert/?key="2024_75466"&descending=true 
            for item in self.db.view("share/Cert", key=year_cert, descending=True):
                doc = item.value
                res.append(doc)

        except Exception as inst:
            msg = "nothing found, receive error: {}".format(inst)
            res = {"error":msg}

        return res

    def save_doc(self, new_doc):
        old_doc = self.get_doc(new_doc["_id"])
        if not 'error' in old_doc:
            new_doc["_rev"] = old_doc["_rev"]

        res = self.db.save(new_doc)
        return res

    def get_countries(self):
        return self.db['000_COUNTRIES'].get('Countries')

    def get_maintainers(self):
        return self.db['000_MAINTAINERS'].get('Maintainers')

    def get_cust_view(self, cust_sign):
        view_con = self.config.get('views').get('customer_by_sign')
        try:
            for item in self.db.view(view_con, key=cust_sign):
                res = item.value

        except Exception as inst:
            msg = "nothing found, receive error: {}".format(inst)
            res = {"error":msg}

        return res
