import re

class Latex:
    latex_subs = (
            (re.compile(r'\\'), r'\\textbackslash'),
            (re.compile(r'([{}_#%&$])'), r'\\\1'),
            (re.compile(r'~'), r'\~{}'),
            (re.compile(r'\^'), r'\^{}'),
            (re.compile(r'"'), r"''"),
            (re.compile(r'\.\.\.+'), r'\\ldots'),
        )
    def __init__(self, app):
  
        texenv = app.jinja_env.overlay()
        texenv.block_start_string = '((*'
        texenv.block_end_string = '*))'
        texenv.variable_start_string = '((('
        texenv.variable_end_string = ')))'
        texenv.comment_start_string = '((#'
        texenv.comment_end_string = '#))'
        texenv.filters['escape_tex'] = self.escape

        self.env = texenv

    def escape(self,value):
        newval = value
        for pattern, replacement in self.latex_subs:
            newval = pattern.sub(replacement, newval)
        return newval
  
    def make_table(self, tables):
        res = {}
        for gas in tables:
            res[gas] = {}
            for result_type in tables[gas]:
                col_tab, i, j = self.make_col_list(tables[gas][result_type])
                res[gas][result_type] = self.col_to_row(col_tab, i, j)
        
        return res

    def make_col_list(self, table):
        col_tab = []
        ncol = 0
        for i, col in enumerate(table):
            ncol = ncol + 1
            nrow = 0
            col_tab.append([])
            for j, row in enumerate(col):
                if row == 'UnitCell':
                    nrow = nrow +1
                    col_tab[i].append({'UnitCell':col[row]})
                if row == 'HeadCell':
                    nrow = nrow +1

                    col_tab[i].append({'HeadCell':col[row]})
                if row == 'Value':
                    for k, v in enumerate(col['Value']):
                        nrow = nrow +1
                        col_tab[i].append({'Value': v})

        return col_tab, ncol, nrow 

    def col_to_row(self, col_tab, ncol, nrow):
        row_tab = []
        for i in range(0, nrow ):
            row_tab.append([])
            for j in range(0, ncol):
                row_tab[i].append(col_tab[j][i])

        return row_tab
