import os
import re
import json

from pprint import pprint

head_re = re.compile("\\\\[a-z]*{([\w\s]*)}")

def rm(h):
     m = head_re.match(h)

     if m:
         return m.groups()[0]
     else:
         return h

def get_dcc_assigment_dict():
    fn = 'cal/dcc-sections-assignment.json'
    with open(fn, 'r') as jf:
            dcc_ass_dict = json.load(jf)

    return dcc_ass_dict

def get_sections(std, device_class, todo_type):
    fn = 'cal/sections-{std}_{device_class}_{todo_type}.json'.format(std=std,device_class=device_class, todo_type=todo_type)
    with open(fn, 'r') as jf:
            section_plan = json.load(jf)

    return section_plan

def get_tex_str_from_file(path, file_name):
    if file_name.endswith("tex"):
        fn = '{path}/{file_name}'.format(path=path, file_name=file_name)
    else:
        fn = '{path}/{file_name}.tex'.format(path=path, file_name=file_name)
    if os.path.isfile(fn):
        with open(fn , 'r', encoding="utf8") as f:
            tex_str = f.read().replace('\n', ' ')
    else:
        tex_str = "missing file: {} (in project cal.git)".format(fn).replace("_","\_")
    return tex_str

def get_all(cert):
    sec_arr = []
    std = get_std(cert)
    device_class = get_device_class(cert)
    todo_type =get_todo_type(cert)
    lang = get_lang(cert)
    section_plan = get_sections(std=std, device_class=device_class, todo_type=todo_type)

    ass_dict = get_dcc_assigment_dict()

    for i, section in enumerate(section_plan):
        path = "templates/latex/{section}".format(section=section)
        file_name = "{lang}-heading".format(lang=lang)
        head = get_tex_str_from_file(path=path, file_name=file_name)
        para_arr = get_paragraphs(section, lang)
        sec_arr.append({"DccAssigment": ass_dict.get(section, "none"),
                        "Heading":head,
                        "Paragraph":para_arr[::-1]})
    return sec_arr

def get_paragraphs(section, lang):
    para_arr = []
    path = "templates/latex/{section}".format(section=section)
    p = re.compile('^(de|en)-para_([0-9]{1,2})-alt_([0-9]{1,2})-(\w*)')
    dir_lst = os.listdir(path)
    dir_lst.sort()
    for i, f in enumerate(dir_lst[::-1]):
        m = p.match(f)
        if m:
            if m.group(1) == lang:
                para_arr.append({"Index": m.group(2),
                                "Alt" : m.group(3),
                                "Type": m.group(4),
                                "Text": get_tex_str_from_file(path=path, file_name=f)
                })
    return para_arr

def get_sub_folders(path):
    folder = "templates/latex/{path}".format(path=path)
    return [f.path for f in os.scandir(folder) if f.is_dir() ]

def get_lang(cert):
    return cert.get("Meta",{}).get("lang")

def get_std(cert):
    """
    The cert. has one main primary standard which provides the struct.
    We simply use the standard at the first issue.
    """
    std = cert.get("Meta",{}).get("std")[0]

    return std.lower()

def get_device_class(cert):
    device_class = cert.get("Device", {}).get("DeviceClass")
    if device_class is not None:
        return device_class.lower()
    else:
        return None

def get_todo_type(cert):
    tdos = cert.get("Meta",{}).get("kind")
    if tdos is not None and len(tdos) > 0:
        if all([tdo == tdos[0] for tdo in tdos]):
            tdo = tdos[0]
        else:
            # implement reduce
            tdo = tdos[0]
    else:
        return None

    return tdo.lower()
