import json
import datetime

import re
from flask import Flask, request, jsonify, send_from_directory, make_response
from flask_cors import CORS

from pprint import pprint

from cal.db import DB
from cal.trans import Trans
import cal.utils as utils
import cal.sections as sections

config = utils.get_config_dict()
git_cmd = utils.git_cmd(config)

app = Flask(__name__, static_url_path='')
CORS(app)
db = DB(config, app)
trans = Trans(config, app, db)

@app.route('/inter/<year_cert>', methods=['GET', 'POST'])
def inter(year_cert):
    r = re.compile("^([0-9]{4})[_-]([0-9]{4,5})$")
    m = r.match(year_cert)
    if m:
        year = m.group(1)
        cert = m.group(2)

        cal_docs = db.get_cal_docs("{}_{}".format(year, cert))
        if not 'error' in cal_docs:
            cal_docs = utils.sort_by_pressure(cal_docs)
            cert_doc = { "_id": "cer-{year}-{cert}".format(year=year, cert=cert),
                                         "Certificate": trans.cert(cal_docs)
                                }
            req = request.get_json()

            if request.method == 'POST':
                res = db.save_doc(cert_doc)
                return res
            else:
                return utils.res_json(cert_doc)
        else:
            return utils.res_json(cal_docs)
    else:
        return utils.return_error("no valid document id given")

@app.route('/dcc/<year_cert>', methods=['GET', 'POST'])
def dcc(year_cert):
    r = re.compile("^([0-9]{4})[_-]([0-9]{4,5})$")
    m = r.match(year_cert)
    if m:
        year = m.group(1)
        cert = m.group(2)

        cal_docs = db.get_cal_docs("{}_{}".format(year, cert))
        if not 'error' in cal_docs:
            cal_docs = utils.sort_by_pressure(cal_docs)
            dcc_doc = { "_id": "dcc-{year}-{cert}".format(year=year, cert=cert),
                         "DCC": trans.dcc(cal_docs)}

            if request.method == 'POST':
                res = db.save_doc(dcc_doc)
                return  utils.res_json(res)
            else:
                return utils.res_json(dcc_doc)
        else:
            return utils.res_json(cal_docs)
    else:
        return utils.return_error("no valid document id given")

@app.route('/cer/latex/<cer_doc_id>', methods=['GET'])
def to_latex(cer_doc_id):
    if cer_doc_id:
        cert_doc = db.get_doc(cer_doc_id)
        if not 'error' in cert_doc:
            res = make_response(trans.cert_to_latex(cert_doc))
            res = utils.set_filename_header(res=res, no=cer_doc_id.split("-")[-1])

            return res

        else:
            return utils.res_json(cert_doc)
    else:
        return utils.return_error("no document id given")

@app.route('/update', methods=['post'])
def update():
    app.logger.debug('hit update')
    req = request.get_json()
    git_cmd.pull()
    app.logger.info("pulled {log}".format(log=git_cmd.log("-n 1")))
    return jsonify({'ok':True})


@app.route('/version', methods=['get'])
def version():
    app.logger.debug('hit version')
    return jsonify({'version': git_cmd.describe()})

if __name__ == '__main__':
    app.run(host=config['server']['host'], port=config['server']['port'])
